/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UPINET_H
#define UPINET_H

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/un.h>

#include <vector>
#include <ostream>

#include <UpTools/UpTimeVal.h>

//forward declarations
struct hostent;
class UpAddress;
class UpNet;
class UpSockAddr;
class UpHost;
class UpSocket;

#ifndef UNIX_PATH_MAX
#define UNIX_PATH_MAX 108
#endif

#define MAX(a,b) ((a<b)?(b):(a))
#define MIN(a,b) ((a<=b)?(a):(b))
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// class UpAdddress for socket addresses

typedef union { in_addr a4; in6_addr a6; } addrUnion;
#define IPBUFSIZE 40

class UpAddress {
private:

	int addrType; // AF_UNSPEC, AF_INET, AF_INET6
	addrUnion upAddr; // for ip4 and ipv6 addresses
	mutable char* buf; // for human readable null terminated string
	friend class UpNet;
	
public:
	UpAddress();
	~UpAddress();
	UpAddress(const in_addr&);	// network byte order
	UpAddress(const in6_addr&);	// network byte order
	UpAddress(const UpAddress&);
	UpAddress(const char*); //human readable null terminated string 
	UpAddress(const char*,size_t len); // human readable not null terminated string 
	UpAddress(const std::string&); //human readable string
	UpAddress(const UpSockAddr&);

	void clear();

	UpAddress& operator=(const in_addr&);	// network byte order
	UpAddress& operator=(const in6_addr&);	// network byte order
	UpAddress& operator=(const UpAddress&);
	UpAddress& operator=(const UpSockAddr&);
	// these three guess family type
	UpAddress& operator=(const char*);        // human readable null terminated string
	UpAddress& operator=(const std::string&); // human readable string
	UpAddress& set(const char*,size_t len);   // human readable not null terminated
	// these three are faster than the three above because they do not guess type
	UpAddress& setf(const char*,size_t len,int fmly); //human readable not null terminated
	UpAddress& setf(const char*,int fmly);            // human readable null terminated
	UpAddress& setf(const std::string&,int fmly);     // human readable string
	bool operator==(const UpAddress&) const;
	bool operator==(const UpSockAddr&) const;
	bool operator==(const UpHost&) const; // es igual a cualquier de UpHost?

	bool operator>(const UpAddress&) const;
	bool operator<(const UpAddress&) const ;
	bool operator>=(const UpAddress&) const;
	bool operator<=(const UpAddress&) const;
	bool operator!=(const UpAddress&) const;

	UpAddress& operator&=(const UpAddress&);
	UpAddress& operator+=(const UpAddress&);
	UpAddress  operator~() const;
	UpAddress  operator+(const UpAddress&) const;
	UpAddress  operator&(const UpAddress&) const;

	void negate(); // negate bits:   (*this) = ~(*this)

	unsigned int maskBitsLen() const;
	void         setMask(unsigned int i,int addrType);
	
	

	size_t hash() const ; 

	int			family() const; 
	size_t		size()	const;	// 4 for ipv4, 16 for ipv6
	const char* c_str()	const;	// string with address in human readable form
	void*			ptr();				// should be casted to in_addr* or in6_addr*
	const void*	ptr()    const;	// should be casted to in_addr* or in6_addr*

	void	write(void* external_addr_buf)	const; // writes binary address to mem. 
		// external_addr_buf should have enough space!
	
	friend std::ostream& operator<<(std::ostream& o, const UpAddress& a);
	friend std::string& operator<<(std::string& o, const UpAddress& a);
};	
//////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// class UpSockAddr for socket addresses

typedef union {
	sockaddr*		a;
	sockaddr_in*	in;
	sockaddr_in6*  in6;
	sockaddr_un*	un;
	} sockaddrPtrUnion;

typedef enum { SOCKADDR_2DOTS, SOCKADDR_SPACE, SOCKADDR_IP } sockaddrFormat;


class UpSockAddr {
private:
	sockaddrPtrUnion sa;
	mutable std::string str;
	mutable sockaddrFormat format;

	void allocAux(int family);
	bool testAssign4(const char* p,unsigned short port);
	bool testAssign6(const char* p,unsigned short port);
	bool testAssignH(const char* p,unsigned short port);
	bool testAssignU(const char* p);
	void assignAux(const UpSockAddr&);
	void assignAux(const char*);
	void assignAux(const char*,unsigned short);
		
public:
	UpSockAddr();
	~UpSockAddr();

	void allocSA(int family=AF_INET);

	int family() const; // AF_UNIX, AF_INET, AF_INET6 or AF_UNSPEC=0
	unsigned short port() const; // if not AF_INET or AF_INET6 returns 0
	
	// Methods for in_addr and in6_addr types
	// the following methods assume network byte order in the ip address
	// but hardware byte order in the port
	UpSockAddr(const in_addr&,unsigned short port);
	UpSockAddr(const in6_addr&,unsigned short port);
	UpSockAddr& operator=(const in_addr&);  // port=0
	UpSockAddr& operator=(const in6_addr&); // port=0
	void set(const in_addr& in, unsigned short port);
	void set(const in6_addr& in6, unsigned short port);
	void set(const UpAddress&, unsigned short port);
	// _addr should be in_addr* or in6_addr* depending on fmly
	void set(const void* _addr, unsigned short port,int fmly=AF_INET);


	// methods for class UpSockAddr
	UpSockAddr(const UpSockAddr&);
	UpSockAddr& operator=(const UpSockAddr&);
	bool operator==(const UpSockAddr&) const; 
	bool operator==(const UpAddress&) const; 
	bool operator==(const UpHost&) const; // igual a uno cualquiera de UpHost?
	bool operator>(const UpSockAddr&) const;
	bool operator<(const UpSockAddr&) const ;
	size_t hash() const ; 

	// Methods for struct sockaddr_xx types
	UpSockAddr(const sockaddr* saddr);
	UpSockAddr(const sockaddr_un* saddr);
	UpSockAddr(const sockaddr_in* saddr);
	UpSockAddr(const sockaddr_in6* saddr);
	UpSockAddr& operator=(const sockaddr* saddr);
	UpSockAddr& operator=(const sockaddr_un* saddr);
	UpSockAddr& operator=(const sockaddr_in* saddr);
	UpSockAddr& operator=(const sockaddr_in6* saddr);
	sockaddr* ptr(); // pointer to use in unix calls
	const sockaddr* ptr() const ; // pointer to use in unix calls
	void write(void*) const; // writes binary sockaddress to mem. 
	size_t size() const; // space needed by the buffer to be used by write(void*)
	
	// methods to use with human readable strings
	// UpSockAddr accepts the following formats in these methods:
	//	"200.44.5.23" "200.44.5.23 80" "200.44.5.23:80"
	// "3ffe:1212::0003" "3ffe:1212::0003 443"
	// "[3ffe:1212::0003]" "[3ffe:1212::0003] 443" "[3ffe:1212::0003]:443"
	// "/tmp/usock"
	// port is in hardware byte order
	// set is faster than operator= because the family is not guessed from data
	UpSockAddr(const char*);
	UpSockAddr(const char*, unsigned short port);
	UpSockAddr& operator=(const char*);
	bool set(const char* addr, unsigned short port,int fmly=AF_INET);
	const char* c_str()       const;	// format "abcd::0001 443"
	const char* c_str_2dots() const;	// format "[abcd::0001]:443"
	const char* c_str_ip()    const;	// format "abcd::001"

	// the same as above but for std::string
	UpSockAddr(const std::string&);
	UpSockAddr(const std::string&, unsigned short port);
	UpSockAddr& operator=(const std::string&);
	bool set(const std::string& addr, unsigned short port,int fmly=AF_INET);
	bool set(const std::string& addr,int fmly=AF_INET); //if !AF_UNIX then port=0 
	const std::string& string()       const;	// format "abcd::0001 443"
	const std::string& string_2dots() const; 	// format "[abcd::0001]:443"
	const std::string& string_ip()    const; 	// format "abcd::001"

	// hostPort Methods accept the same formats as before, and also domains,
	// but do not allow unix paths:
	//	"200.44.5.23" "200.44.5.23 80" "200.44.5.23:80"
	// "3ffe:1212::0003" "3ffe:1212::0003 443"
	// "[3ffe:1212::0003]" "[3ffe:1212::0003] 443" "[3ffe:1212::0003]:443"
	// "www.palermo.edu" "www.palermo.edu 80" "www.palermo.edu:80"
	bool hostPort(const char* hostAddr, unsigned short port);
	bool hostPort(const char* hostAddr);
	bool hostPort(const std::string& hostAddr, unsigned short port);
	bool hostPort(const std::string& hostAddr);

	// used to print UpSockAddr to ostreams	
	friend std::ostream& operator<<(std::ostream& o, const UpSockAddr& upa);
	friend std::string& operator<<(std::string& o, const UpSockAddr& upa);
};	
//////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Class UpHost

#define HOSTBUFSIZE 256 // used for GNU GLIBC2 gethostbyname_r()

class UpHost {
private:
	std::string              _hostName;		// host name
	std::vector<std::string> _aliases;		// container for the aliases
	std::vector<UpAddress>   _addresses;	// container for the addresses
	int addrType;									// AF_UNIX, AF_INET, AF_INET6    
	size_t addrLen;								// depends on the address type
	bool lookupOk;									// did the host lookup succeed?
	int lookupErrno;							// error number in case the lookup fail


	bool init(const char* host);
	void init(const hostent* h);

public:
	UpHost();
	UpHost(const char*);
	UpHost(const std::string&);
	UpHost(const UpHost&);
	UpHost& operator=(const char*);
	UpHost& operator=(const std::string&);
	UpHost& operator=(const UpHost&);
	bool operator==(const UpHost&) const; // igual a cualquiera
	bool operator==(const UpAddress&) const; // igual a cualquiera
	bool operator==(const UpSockAddr&) const; // igual a cualquiera

	bool										status() const;
	int										errNumber() const;
	const std::string&					errPhrase() const;

	int 										family() const;
	size_t 									addrSize() const;
	const std::string&					hostName() const;
	const std::vector<std::string>&	aliases() const;
	const std::vector<UpAddress>&		addresses() const;
	const UpAddress&						address() const;

	friend std::ostream& operator<<(std::ostream& o, const UpHost& h);
	friend std::string& operator<<(std::string& o, const UpHost& h);
};


////////////////////////////////////////////////////////////////////////	
////////////////////////////////////////////////////////////////////////	
// Class UpSocket


#define DEFAULTQUEUESIZE	5  // old BSD's could listen up to 5 incoming max

class UpSocket {
private:
	int sd;
	UpSockAddr localSockAddr,remoteSockAddr;
	int family;
	//flags
	bool nonBlocking;	// default false
	bool direct;		// default false
	bool async;			// default false
	UpTimeVal sendTimeout; // default inifinite
	UpTimeVal recvTimeout; // default inifinite
	// administrative data
	int  sockType;		// default SOCK_STREAM;
	// internal methods
	bool bind();		// internal bind method
	bool connect();	// internal connect method
	bool setFlags();	// internal setFlags method

public:
	UpSocket();
	~UpSocket(); // closes socket before destroying
//	virtual ~UpSocket();

	// error handling
	const std::string& getLastError();

	// set flags	
	void setNonBlocking(bool);	// sets non-blocking operations
	void setDirect(bool);	// sets direct operations (flushes immedialtely)
	void setAsync(bool);		// sets async operations (SIGIO when data available)
	// set maximum wait time for connect(...), write(...) or send(...)
	void setSendTimeout(const UpTimeVal& tv);
	// set maximum wait time for accept(...), read(...) or recv(...)
	void setRecvTimeout(const UpTimeVal& tv);


	// set options
	// after calling fd(), bind(...) or connect(...) these methods cannot change
	// sockType and return false if sockType differs from what was requested
	bool setDatagram();		// sets sockType = SOCK_DGRAM   (udp for ip sockets)
	bool setStream();			// sets sockType = SOCK_STREAM  (tcp for ip sockets)

	// administrative
	void showFcntlFlags();
	int  fd();
	const UpSockAddr& getLocalSockAddr();
	const UpSockAddr& getRemoteSockAddr();

	// for clients or servers (not necessary for clients)
	bool bind(const UpSockAddr& localSA);
	bool bind(const sockaddr* localSA);
	bool bind(const std::string& localAddr);
	bool bind(const std::string& localAddr,unsigned short localPort);
	bool bind(const char*);
	bool bind(const char*,unsigned short port);
	// these two acept only AF_INET and AF_INET6
	bool bindInterface(const std::string& interf,unsigned short localPort,int _family=AF_INET);
	bool bindInterface(const char* interf, unsigned short localPort,int _family=AF_INET);

	// for server sockets
	bool listen();
	bool listen(int queueSize);

	// connected socket is initialized with the new accepted connection
	bool accept(UpSocket& connectedSocket); 

	// for client sockets
	// generally used by stream sockets
	bool connect(const sockaddr* remoteSA);
	bool connect(const UpSockAddr& remoteSA);
	bool connect(const std::string& remoteAddr);
	bool connect(const std::string& remoteAddr, unsigned short remoteP);
	bool connect(const char* remoteAddr);
	bool connect(const char* remoteAddr, unsigned short remoteP);

	// for all sockets
	void close();

	// for all sockets
	// generally used by stream sockets
	int write(const std::string&);
	int write(const char*buf ,size_t len);
	int writeUrg(const std::string&);
	int writeUrg(const char* buf, size_t len);

	// for ipv4 or ipv6 sockets
	// generally used by datagram sockets
	int sendto(const std::string&, const UpSockAddr& remoteSA);
	int sendto(const char*,size_t len, const UpSockAddr& remoteSA);
	int sendtoUrg(const std::string& data, const UpSockAddr& remoteSA);
	int sendtoUrg(const char* data, size_t len, const UpSockAddr& remoteSA);
	
	// generally used by stream sockets
	int read(char*,size_t len);
	int readUrg(char*,size_t len);
	// int read(std::string&);
	// int readUrg(std::string&);

	// remoteSA is rewritten with de data of the incoming packet
	// generally used in datagram sockets
	// if data does not fit in the buf, the extra bytes are discarded
   int recvFrom(char* buf, size_t len, UpSockAddr& remoteSA);
   int recvFromUrg(char* buf, size_t len, UpSockAddr& remoteSA);
	// int recvFrom(std::string&, UpSockAddr& remoteSA);
   // int recvFromUrg(std::string& data, UpSockAddr& remoteSA);

};


////////////////////////////////////////////////////////////////////////

typedef enum {
	UPNR_BEFORE,
	UPNR_AFTER,
	UPNR_SUPERNET,
	UPNR_SUBNET,
	UPNR_EQUAL,
	UPNR_INVALID_RELATION
} UpNetRelation;

class UpNet {
public:

	UpNet();
	UpNet(const UpNet&);
	UpNet(const UpAddress& baseAddr,const UpAddress& mask);
	UpNet(const UpAddress& baseAddr,unsigned int maskLen);
	UpNet(in_addr baseAddr,in_addr mask);   // net order
	UpNet(const in6_addr& baseAddr,const in6_addr& mask);   // net order
	UpNet(const std::string& net);

	void set(const UpNet&);
	void set(const UpAddress& baseAddr,const UpAddress& mask);
	void set(const UpAddress& baseAddr,unsigned int maskLen);
	void set(const in_addr& baseAddr,in_addr& mask);   // net order
	void set(const in6_addr& baseAddr,in6_addr& mask);   // net order
	void set(const std::string& net);

	UpNet& operator=(const UpNet&);
	UpNet& operator=(const std::string&);

	UpNetRelation check(const UpNet& net) const;
	UpNetRelation check(const UpAddress& addr) const;
	UpNetRelation check(const UpSockAddr& addr) const;
	UpNetRelation check(const in_addr& addr) const;   // net order
	UpNetRelation check(const in6_addr& addr) const;   // net order

	bool includes(const UpNet& net) const;
	bool includes(const UpAddress& addr) const;
	bool includes(const UpSockAddr& addr) const;
	bool includes(const in_addr& addr) const;   // net order
	bool includes(const in6_addr& addr) const;   // net order

	unsigned int getMaskLen() const;
	const UpAddress& getBase() const;
	const UpAddress& getMask() const;
	int family() const;

	friend std::ostream& operator<<(std::ostream& o,const UpNet& net);
	friend std::string& operator<<(std::string& o,const UpNet& net);

	static const std::string& relationString(UpNetRelation);

private:
	UpAddress ip;
	UpAddress m;
	void checkPrivateData();
	static std::string relString[];
};





#endif

#include <UpTools/UpInet.inl>
