/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/***************************************************************************
Why use UPLOG instead of syslog:

	- UPLOG allows to write to syslogd or directly to a file.
	- UPLOG allows to print thread id of the calling thread.
	- UPLOG does not execute function calls for any parameters if logLevel is
	  lower than the UPLOG level. For example:
			UPLOG(LOG_INFO,"thing happend: %d %d",myCount(),myAtoms());
			myCount() and myAtoms() will not be executed if logLevel was set to
			LOG_WARNING. (This is the reason why UPLOG is a macro instead of a
			function).
		
***************************************************************************
When to use UPDEBUG(...) instead of UPLOG(LOG_DEBUG,...):

	UPDEBUG(...) disappears from the source code when not compiled with -DDEBUG.
	So it is useful for development-time-only logging.

***************************************************************************
To check that program execution goes through specific points of the program,
just throw a X or a UPMARK to the line of interes. (both need -DDEBUG):

	UPMARK elemPtr = mapFind(elemKey);
	UPMARK n = processElem(elemPtr);
	X		 if(n>0) {
	X			doProc();
			 } else {
	X			doOtherProc();
			 }
	X

***************************************************************************/

#ifndef UPLOG_H
#define UPLOG_H


#include <syslog.h>
#include <UpTools/UpLog.inl>

extern "C" {


/***************************************************************************/
/* In order to log to a file use upOpenLogFile. */
#ifndef upOpenLogFile
#define upOpenLogFile(ident,options,filename,logLevel) \
					upOpenLogFileInternal(ident,options,filename,logLevel,0)
#endif

/*	In order to log to syslog use upOpenLog instead. */
#define upOpenLog(ident,options,facility,logLevel) \
					upOpenLogInternal(ident,options,facility,logLevel)

/* upOpenLog and upOpenLogFile parameters:

 	const char* ident: text prepended to every message.
   int options: an OR of any of these: LOG_CONS, LOG_NDELAY, LOG_NOWAIT,
		LOG_ODELAY, LOG_PERROR, LOG_PID, LOG_THID.
		(LOG_THID:log thread id, works only with upOpenLogFile, requires -lpthread)
	int facility: any of these:  LOG_AUTHPRIV, LOG_CRON, LOG_DAEMON, LOG_FTP, LOG_KERN,
		LOG_LOCAL[0-7], LOG_LPR, LOG_MAIL, LOG_NEWS, LOG_SYSLOG,
		LOG_USER(default), LOG_UUCP.
	int logLevel: only messages with level higher or equal to logLevel will be logged
		any of these: LOG_EMERG, LOG_ALERT, LOG_CRIT, LOG_ERR, LOG_WARNING,
		LOG_NOTICE, LOG_INFO, LOG_DEBUG.
	const char* filename: pathname of file to output message.
*/


/* to close log */
void upCloseLog();

/****************************************************************************/

/* UPLOG(level,...) is the main log macro */
#define UPLOG(level,...) _UPLOG_(level, __VA_ARGS__ )

/* UPDEBUG(...) macros disappear from the source code when not compiling 
 * with -DDEBUG. */
#ifdef DEBUG
	#define UPDEBUG(...) _UPLOG_(LOG_DEBUG, __VA_ARGS__ )
#else
	#define UPDEBUG(...)
#endif

/* user may redefine the max log line before including UpLog.h */
#ifndef MAXLOGLINE
	#define MAXLOGLINE 1024
#else
	#if MAXLOGLINE < 256
		#define MAXLOGLINE 256
	#endif
#endif


/* to log when execution goes through specific program points */
#define UPMARK UPDEBUG("debug-mark %s line %d %s",__FILE__,__LINE__,__QQFUNC__);
#define X UPMARK

/* useful compiled predefined macros: */
/* __FILE__, __DATE__, __TIME__, __LINE__, __func__, __PRETTY_FUNCTION__ */


}
	

#endif
