/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPTHREADSPECIFIC_H
#define UPTHREADSPECIFIC_H

#include <pthread.h>

/////////////////////////////////////////////////////////////////////////////
// template UpThreadSpecificPtr
//
// This template points to objects specific to the calling thread.
// Destruction of the objects must be handled by the programmer.
// To automatically handle the destruction of the thread specific objects,
// use UpThreadSpecificSmartPtr.

template<typename T=void>
class UpThreadSpecificPtr {
public:
	UpThreadSpecificPtr();
	UpThreadSpecificPtr(const T* ptr);
	UpThreadSpecificPtr(const UpThreadSpecificPtr<T>& sptr);
	// destroys semaphore
	~UpThreadSpecificPtr();
	
	// operadores de asignacion
	UpThreadSpecificPtr<T>& operator=(const T* ptr);
	UpThreadSpecificPtr<T>& operator=(const UpThreadSpecificPtr<T>& sptr);

	// C++ considera un error la conversion implicita de  un void* a cualquier
	// otro tipo de puntero.
	// Pero C++ convierte en forma automatica cualquier puntero en un void* .
	// Para que pueda hacer esto con nuestra esta necesitamos el operador
	// de conversion.
	operator	void*();

	// conversión a puntero comun
	operator T*();

	// operadores utiles
	T&				operator*();
	const T&		operator*() const;
	T*				operator->();
	const T*		operator->() const;


	// operadores de comparacion
	bool	operator!() const;
	bool	operator==(const T* ptr) const;
	bool	operator!=(const T* ptr) const;
	bool	operator==(const UpThreadSpecificPtr<T>& sptr) const;
	bool	operator!=(const UpThreadSpecificPtr<T>& sptr) const;
	bool	operator<(const UpThreadSpecificPtr<T>& sptr) const;
private:
	pthread_key_t k;
	T* get() const; 
	void set(const T*);
};

/////////////////////////////////////////////////////////////////////////////
// template UpThreadSpecificSmartPtr
//
// This template points to objects specific to the calling thread.
// Destruction of the objects is automatically handled (for example when
// the thread is finished).
// Objects pointed must derive from class UpLinkable
// TODO:

template<typename T=void>
class UpThreadSpecificSmartPtr {
public:
	UpThreadSpecificSmartPtr();
	UpThreadSpecificSmartPtr(const T* ptr);
	UpThreadSpecificSmartPtr(const UpThreadSpecificSmartPtr<T>& sptr);
	// destroys semaphore
	~UpThreadSpecificSmartPtr();
	
	// operadores de asignacion
	UpThreadSpecificSmartPtr<T>& operator=(const T* ptr);
	UpThreadSpecificSmartPtr<T>& operator=(const UpThreadSpecificSmartPtr<T>& sptr);

	// C++ considera un error la conversion implicita de  un void* a cualquier
	// otro tipo de puntero.
	// Pero C++ convierte en forma automatica cualquier puntero en un void* .
	// Para que pueda hacer esto con nuestra esta necesitamos el operador
	// de conversion.
	operator	void*();

	// conversión a puntero comun
	operator T*();

	// operadores utiles
	T&				operator*();
	const T&		operator*() const;
	T*				operator->();
	const T*		operator->() const;


	// operadores de comparacion
	bool	operator!() const;
	bool	operator==(const T* ptr) const;
	bool	operator!=(const T* ptr) const;
	bool	operator==(const UpThreadSpecificSmartPtr<T>& sptr) const;
	bool	operator!=(const UpThreadSpecificSmartPtr<T>& sptr) const;
	bool	operator<(const UpThreadSpecificSmartPtr<T>& sptr) const;
private:
	pthread_key_t k;
	T* get() const; 
	void set(const T*);
	void hold(const T*);
	void release(const T*);
};

#include <UpTools/UpThreadSpecific.inl>

#endif
